package art;

public class Animation
{
    protected final static int LOOP_NONE = 0;
    protected final static int LOOP_FORTH = 1;
    protected final static int LOOP_BACK_FORTH = 2;
    protected final static int LOOP_BACK = 3;

    protected final static int LOOP_REPEAT_NONE = 0;
    protected final static int LOOP_REPEAT_ONCE = 1;
    protected final static int LOOP_REPEAT_FOREVER = 2;

    private final static int LOOP_DIRECTION_FORTH = 0;
    private final static int LOOP_DIRECTION_BACK = 1;

    private int maxCell = 1;
    private final static int MIN_CELL = 1;

    protected int cell = MIN_CELL;
    protected int loopBehavior = LOOP_FORTH;
    protected int loopRepeatBehavior = LOOP_REPEAT_FOREVER;

    private boolean doDelay = false;
    private int delayValue = 0;
    private int delayMax = 2;

    private boolean didRepeatOnce = false;

    private int loopDirection = LOOP_DIRECTION_FORTH;

    public void reset()
    {
        didRepeatOnce = false;
        cell = MIN_CELL;
    }

    public void setLoopBehavior(int _loopBehavior)
    {
        loopBehavior = _loopBehavior;
    }

    public void setLoopRepeatBehavior(int _loopRepeatBehavior)
    {
        loopRepeatBehavior = _loopRepeatBehavior;
    }

    public void calculateCell()
    {
        if ( !inDelay() )
            calculateNextCell();
    }

    public int getCell()
    {
        return cell;
    }

    public int getCalculatedCell()
    {
        calculateCell();
        return cell;
    }

    public void setDelay(int _delayMax)
    {
        if ( _delayMax != 0 )
        {
            doDelay = true;
            delayMax = _delayMax;
        }
        else
            removeDelay();
    }
    public void removeDelay()
    {
        doDelay = false;
        delayMax = 0;
    }

    public void setMaxCell(int _maxCell)
    {
        maxCell = _maxCell;
    }

    public int getInvertedCell()
    {
        return maxCell - cell;
    }

    private boolean inDelay()
    {
        boolean isInDelay = false;

        if ( doDelay )
        {
            if ( ++delayValue <= delayMax )
                isInDelay = true;
            else
            {
                isInDelay = false;
                delayValue = 0;
            }
        }
        else
            isInDelay = false;

        return isInDelay;            
    }

    private void calculateNextCell()
    {
        nextCell();
        adjustCell();
    }

    private void adjustCell()
    {
        if ( cell < MIN_CELL )
            adjustCellBelowMin();
        else if ( cell > maxCell )
            adjustCellAboveMax();
    }

    private void adjustCellBelowMin()
    {
        switch ( loopRepeatBehavior )
        {
            case LOOP_REPEAT_NONE:
                cell = MIN_CELL;
                break;
            case LOOP_REPEAT_ONCE:
                if ( didRepeatOnce )
                    cell = MIN_CELL;
                else
                {
                    adjustBelowMinAccordingToBehavior();
                    didRepeatOnce = true;
                }
                break;
            case LOOP_REPEAT_FOREVER:
                adjustBelowMinAccordingToBehavior();
                break;
        }
    }

    private void adjustCellAboveMax()
    {
        switch ( loopRepeatBehavior )
        {
            case LOOP_REPEAT_NONE:
                cell = MIN_CELL;
                break;
            case LOOP_REPEAT_ONCE:
                if ( didRepeatOnce )
                    cell = MIN_CELL;
                else
                {
                    adjustAboveMaxAccordingToBehavior();
                    didRepeatOnce = true;
                }
                break;
            case LOOP_REPEAT_FOREVER:
                adjustAboveMaxAccordingToBehavior();
                break;
        }
    }

    private void adjustAboveMaxAccordingToBehavior()
    {
        if ( loopBehavior == LOOP_FORTH )
        {
            cell = MIN_CELL;
        }
        else if ( loopBehavior == LOOP_BACK_FORTH )
        {
            cell = maxCell;
            reverseLoopDirection();
        }
    }

    private void adjustBelowMinAccordingToBehavior()
    {
        if ( loopBehavior == LOOP_BACK )
        {
            cell = maxCell;
        }
        else if ( loopBehavior == LOOP_BACK_FORTH )
        {
            cell = MIN_CELL;
            reverseLoopDirection();
        }
    }

    private void reverseLoopDirection()
    {
        loopDirection =
            loopDirection == LOOP_DIRECTION_FORTH ?
                LOOP_DIRECTION_BACK : LOOP_DIRECTION_FORTH;
    }

    private void nextCell()
    {
        switch ( loopBehavior )
        {
            case LOOP_NONE:
                break;
            case LOOP_FORTH:
                cell++;
                break;
            case LOOP_BACK:
                cell--;
                break;
            case LOOP_BACK_FORTH:
                cell = loopDirection == LOOP_DIRECTION_FORTH ?
                    cell+1 : cell-1;
                break;
        }
    }

}