package game;
import java.awt.*;

public class Camera extends IntelligentMovingThing
{
    final shape.Size targetMargin = new shape.Size(100,100);
    private ViewPort viewPort = new ViewPort();

    public Camera(shape.Size viewSize)
    {
        super(10,10,255,255,255);
        viewPort.setSize(viewSize);
        setMarginForBorder( viewSize.getHalf() );
    }
    public Camera(int quadviewSize)
    {
        super(10,10,255,255,255);
        shape.Size viewSize = new shape.Size(quadviewSize);
        viewPort.setSize( viewSize );
        setMarginForBorder( viewSize.getHalf() );
    }

    protected int getFollowAdjustSpeed()
    {
        return 10;
    }
    protected int getFollowMaxSpeed()
    {
        return 25;
    }
    protected double getBrakeAdjustSpeed()
    {
        return .5;
    }
    protected double getBrakeAt() // 
    {
        return 9;
    }

    public int getViewLeft()
    {
        return viewPort.getLeft( (int) x );
    }
    public int getViewTop()
    {
        return viewPort.getTop( (int) y );
    }
    public int getViewRight()
    {
        return viewPort.getRight( (int) x );
    }
    public int getViewBottom()
    {
        return viewPort.getBottom( (int) y );
    }
    public int getViewWidth()
    {
        return viewPort.getWidth();
    }
    public int getViewHeight()
    {
        return viewPort.getHeight();
    }

    public void act(Ground ground, shape.Point targetPosition)
    {
        follow(targetPosition, targetMargin);
        bump(ground);
        move(ground);
    }
}

class ViewPort
{
    private int
        upward = 0, downward = 0,
        leftward = 0, rightward = 0;

    public ViewPort(shape.Size size)
    {
        setSize(size);
    }
    public ViewPort()
    {
    }

    public void setSize(shape.Size size)
    {
        leftward = (int) size.width / 2;
        upward = (int) size.height / 2;
        rightward = (int) size.width / 2;
        downward = (int) size.height / 2;
    }

    public shape.Rectangle getRectangle(int x, int y)
    {
        return new shape.Rectangle(
            x-leftward, y-upward,
            x+rightward, y+downward
            );
    }

    public int getWidth()
    {
        return leftward+rightward;
    }
    public int getHeight()
    {
        return upward+downward;
    }

    public int getLeft(int x)
    {
        return x-leftward;
    }
    public int getTop(int y)
    {
        return y-upward;
    }
    public int getRight(int x)
    {
        return x+rightward;
    }
    public int getBottom(int y)
    {
        return y+downward;
    }

}
